import {ChangeDetectorRef, Component, OnInit} from '@angular/core';
import {ActivatedRoute, Router} from '@angular/router';
import {SpeciesAnalysis} from 'src/app/models/SpeciesAnalysis';
import {DataService} from 'src/app/services/data.service';
import {SLFDatabase} from "../../models/SLFDatabase";
import {ModelAnalysis} from "../../models/ModelAnalysis";
import {RegionModelAnalysis} from "../../models/RegionModelAnalysis";
import {RegionTreeDistance} from "../../models/RegionTreeDistance";
import {FamilyAnalysis} from "../../models/FamilyAnalysis";

class SpeciesRow {
  public readonly regionName: string;
  public readonly modelName: string;
  public readonly predicted?: RegionModelAnalysis;
  public readonly putative?: RegionModelAnalysis;

  constructor(
    regionName: string, modelAnalysis: ModelAnalysis
  ) {
    this.regionName = regionName;
    this.modelName = modelAnalysis.name;
    this.predicted = modelAnalysis.predicted;
    this.putative = modelAnalysis.putative;
  }

  public hasPredictedAnalysis() {
    return this.predicted !== null && this.predicted !== undefined;
  }

  public hasPutativeAnalysis() {
    return this.putative !== null && this.putative !== undefined;
  }

  public hasBothAnalysis() {
    return this.hasPredictedAnalysis() && this.hasPutativeAnalysis();
  }

  private getRegionModelAnalysis(predictedSelected: boolean): RegionModelAnalysis {
    if (predictedSelected) {
      return <RegionModelAnalysis>this.predicted;
    } else {
      return <RegionModelAnalysis>this.putative;
    }
  }

  public getMinTreeDistance(predictedSelected: boolean): RegionTreeDistance[] {
    const regionModelAnalysis = <RegionModelAnalysis>this.getRegionModelAnalysis(predictedSelected);
    return regionModelAnalysis.minTreeDistances;
  }

  public getIsoelectricPoints(predictedSelected: boolean): number[] {
    const regionModelAnalysis = <RegionModelAnalysis>this.getRegionModelAnalysis(predictedSelected);
    return regionModelAnalysis.isoelectricPoints;
  }
}

@Component({
  selector: 'slf-result',
  templateUrl: './result.component.html',
  styleUrls: ['./result.component.scss']
})
export class ResultComponent implements OnInit {
  public readonly id: string;
  public readonly familyName: string;
  private readonly speciesName: string;
  private database?: SLFDatabase;
  private familyAnalysis?: FamilyAnalysis;
  private speciesAnalysis?: SpeciesAnalysis;
  public speciesAnalysisNames: string[];
  public rowData: SpeciesRow[];
  private checkedAnalysis: Map<string, Map<string, boolean>>;
  public readonly displayedColumns: string[] =
    ['predicted', 'region', 'model', 'min_tree_distance', 'isoelectric_point'];

  constructor(
    private readonly changeDetectorRef: ChangeDetectorRef,
    private readonly route: ActivatedRoute,
    private readonly router: Router,
    private readonly dataService: DataService
  ) {
    const paramMap = this.route.snapshot.paramMap;
    if (!(paramMap.has('family') && paramMap.has('id'))) {
      this.router.navigate(['/']);
      throw new Error('Missing familyName and id');
    }
    this.familyName = <string>paramMap.get('family');
    this.id = <string>paramMap.get('id');
    this.speciesName = this.id.substring(0, this.id.indexOf("_" + this.familyName));
    this.speciesAnalysisNames = [];
    this.rowData = [];
    this.checkedAnalysis = new Map<string, Map<string, boolean>>();
  }

  public ngOnInit(): void {
    this.dataService.getFamily(this.familyName)
      .subscribe(familyAnalysis => {
        this.familyAnalysis = familyAnalysis;
        this.speciesAnalysis = familyAnalysis.speciesAnalysis[this.id];
        this.speciesAnalysisNames = Object.keys(familyAnalysis.speciesAnalysis)
          .filter(speciesName => speciesName.includes(this.speciesName));
        this.rowData = Object.values(this.speciesAnalysis.regionAnalysis) // Array de RegionAnalysis
          .flatMap(region => Object.values(region.modelAnalysis)  // Array ModelAnalysis
            .flatMap(modelAnalysis => {
              const rowData = new SpeciesRow(region.name, modelAnalysis);
              if (!this.checkedAnalysis.has(rowData.regionName)) {
                this.checkedAnalysis.set(rowData.regionName, new Map<string, boolean>());
              }
              // @ts-ignore
              this.checkedAnalysis.get(rowData.regionName).set(rowData.modelName, rowData.hasPredictedAnalysis());
              return rowData;
            }) // Array ModelAnalysis
          );
      });
  }

  public onChangeResults(value: string): void {
    this.router.navigate(['species', this.familyName, value])
      .then(() => window.location.reload());
  }

  public isChecked(analysis: SpeciesRow): boolean {
    const modelChecked = this.checkedAnalysis.get(analysis.regionName);
    return modelChecked !== undefined && modelChecked.get(analysis.modelName) === true;
  }

  public onToggleRow(analysis: SpeciesRow): void {
    const checked = this.isChecked(analysis);
    // @ts-ignore
    this.checkedAnalysis.get(analysis.regionName).set(analysis.modelName, !checked);
  }
}
